# Generated by roxygen2: do not edit by hand

export(file_edit)
export(get_usgs_gitlab_url)
export(tar_init)
export(use_changelog_usgs)
export(use_code_json_usgs)
export(use_code_of_conduct_usgs)
export(use_contributing_usgs)
export(use_disclaimer_approved_usgs)
export(use_disclaimer_provisional_usgs)
export(use_gitignore_usgs)
export(use_gitlab_mr_template_usgs)
export(use_license_usgs)
export(use_project_usgs)
export(use_readme_rmd_usgs)
export(use_readme_usgs)
