% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/use_project_usgs.R
\name{use-file-usgs}
\alias{use-file-usgs}
\alias{use_gitignore_usgs}
\alias{use_readme_usgs}
\alias{use_readme_rmd_usgs}
\alias{use_license_usgs}
\alias{use_code_json_usgs}
\alias{use_disclaimer_provisional_usgs}
\alias{use_disclaimer_approved_usgs}
\alias{use_code_of_conduct_usgs}
\alias{use_contributing_usgs}
\alias{use_changelog_usgs}
\alias{use_gitlab_mr_template_usgs}
\title{Add individual USGS project files to a directory}
\usage{
use_gitignore_usgs(home, additions = NULL, open = rlang::is_interactive())

use_readme_usgs(home, open = rlang::is_interactive())

use_readme_rmd_usgs(home, open = rlang::is_interactive())

use_license_usgs(home, open = rlang::is_interactive())

use_code_json_usgs(home, open = rlang::is_interactive())

use_disclaimer_provisional_usgs(home, open = rlang::is_interactive())

use_disclaimer_approved_usgs(home, open = rlang::is_interactive())

use_code_of_conduct_usgs(home, open = rlang::is_interactive())

use_contributing_usgs(home, repo_url = NULL, open = rlang::is_interactive())

use_changelog_usgs(home, open = rlang::is_interactive())

use_gitlab_mr_template_usgs(home, open = FALSE)
}
\arguments{
\item{home}{chr, root directory of targets project. To use the current
working directory, use  \code{"."}.}

\item{additions}{chr vector, other files/directories to be added to
.gitignore}

\item{open}{lgl; whether to open the file for interactive editing}

\item{repo_url}{chr, URL to Git repository. If \code{NULL} (default) a generic
URL will be provided as a link to the repository in CONTRIBUTING.md.
Otherwise, the issue page for \code{repo_url} will be used as the link.}
}
\value{
\code{NULL} invisibly
}
\description{
Creates common project files like .gitignore, README.md. LICENSE.md,
etc.#'
}
\examples{
tmp <- tempdir()

use_gitignore_usgs(home = tmp,
                   additions = c("excluded_file.R",
                                 "excluded_dir",
                                 "*excluded_pattern*"))
# here are the contents of the .gitignore:
cat(readLines(file.path(tmp,".gitignore")), sep = "\n")

unlink(tmp, recursive = TRUE, force = TRUE)

}
