% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/use_project_usgs.R
\name{use_project_usgs}
\alias{use_project_usgs}
\title{Set up a USGS project directory}
\usage{
use_project_usgs(
  home,
  gitignore_additions = NULL,
  readme_type = c("md", "rmd"),
  disclaimer_type = c("provisional", "approved"),
  repo_url = get_usgs_gitlab_url("origin"),
  use_mr_template = TRUE,
  open = rlang::is_interactive()
)
}
\arguments{
\item{home}{chr, root directory of targets project. To use the current
working directory, use  \code{"."}.}

\item{gitignore_additions}{chr vector, other files/directories to be added to
.gitignore or \code{NULL} (default) for no additions.}

\item{readme_type}{chr, either "md" to create README as a markdown file
(README.md; default) or "rmd" to create it as an R markdown file
(README.Rmd). Using an Rmd file will allow you to include R code and output
in your README.md.}

\item{disclaimer_type}{chr, either "provisional" (default) to include the
provisional software disclaimer statement or "approved" for the approved
disclaimer statement. It is important to only include the approved
disclaimer statement if your software is an approved software release.}

\item{repo_url}{chr, URL to Git repository. If \code{NULL} (default) a generic
URL will be provided as a link to the repository in CONTRIBUTING.md.
Otherwise, the issue page for \code{repo_url} will be used as the link.}

\item{use_mr_template}{lgl, Should GitLab Merge Request template be included
in project repository. Default is \code{TRUE}.}

\item{open}{lgl; whether to open the files for interactive editing.}
}
\value{
\code{NULL} invisibly
}
\description{
Wraps the other \code{use_*_usgs} functions to initialize
multiple files in one directory.
}
\examples{
tmp <- tempdir()
unlink(tmp, recursive = TRUE, force = TRUE)
dir.create(tmp)

use_project_usgs(
  home = tmp,
  repo_url = "https://code.usgs.gov/water/computational-tools/mortar"
)
list.files(tmp)

# start over
unlink(tmp, recursive = TRUE, force = TRUE)
dir.create(tmp)

# creates README.Rmd and DISCLAIMER_APPROVED instead
use_project_usgs(
  home = tmp,
  readme_type = "rmd",
  disclaimer_type = "approved",
  repo_url = "https://code.usgs.gov/water/computational-tools/mortar"
)
list.files(tmp)

# Clean up temp files
unlink(tmp, recursive = TRUE, force = TRUE)

}
