% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/use_project_usgs.R
\name{get_usgs_gitlab_url}
\alias{get_usgs_gitlab_url}
\title{Get the HTTPS URL for the git remote of a repo at code.usgs.gov}
\usage{
get_usgs_gitlab_url(remote_name = "origin")
}
\arguments{
\item{remote_name}{chr; name of the git remote. The default is "origin".}
}
\value{
a character sting of the HTTPS URL to the remote repo
}
\description{
Get the HTTPS URL for the git remote of a repo at code.usgs.gov
}
\examples{
\dontrun{
# Will fail if working directory is not in a git repo
get_usgs_gitlab_url("origin")
}
}
